from octo.handler.command import Command


class StartApp(Command):
    """Create a django app inside the app/"""

    def __init__(self):
        super().__init__()
        self.use_django = False

    def handle(self):
        # === import =====================================
        import subprocess
        import os
        from octo.base import BASE_DIR as octo_dir, _error_structure
        from colorama import Fore, Style, init

        init()
        # === Logic ======================================
        APP_DIR = os.getcwd() + "/app"

        if not os.path.exists(APP_DIR):
            raise ValueError(Fore.RED + _error_structure + Style.RESET_ALL)

        os.chdir(APP_DIR)

        if len(self._argv) > 2:
            app_name = self._argv[2]
            subprocess.run(
                [
                    "django-admin",
                    "startapp",
                    f"{app_name}",
                    f"--template={octo_dir}/conf/app_template",
                ]
            )
        else:
            print(
                Fore.RED
                + "\n"
                + "You must enter the application name"
                + Fore.GREEN
                + "\n"
                + "octo startapp app_name"
                + Style.RESET_ALL
            )



from octo.handler.oauth import OAuthHandler
import requests
from django.conf import settings


class OAuthGoogle(OAuthHandler):
    def get_service_url(self):
        return "https://www.googleapis.com/oauth2/v3/userinfo"

    def validate_access_token(self) -> bool:
        validation_url = "https://www.googleapis.com/oauth2/v3/tokeninfo"

        response = requests.get(
            validation_url, params={"access_token": self.access_token}
        )

        if response.status_code == 200:
            token_info = response.json()
            try:
                client_ids = settings.OCTO_OAUTH.get("google")
            except AttributeError:
                raise ValueError(
                    "The 'OCTO_OAUTH' setting is not properly configured. Please ensure 'OCTO_OAUTH' is defined in your settings and contains a valid key for 'google'."
                )
            if token_info["aud"] in client_ids:
                return True

        return False



import sys
import os
import django


class Command:
    """Represents a base command class with hooks and setup for Django environment."""

    _hooks = {
        "--no-error": "no_error",
    }

    def __init__(self):
        self._argv = sys.argv
        self.django_run = False  # Determines whether Django is running or not, it is automatically changed when it is running
        self.use_django = True  # Determines whether Django is required or not
        self.debug = True  # It expresses whether error messages will be printed or not, and this can be controlled via a hook called --no-error

    def set_hooks(self, new_hook: dict) -> None:
        """Adds new hooks to the hook dictionary after checking for duplicates.

        Args:
            new_hook (dict): A dictionary of new hooks to add to the hook dictionary.

        Raises:
            ValueError: If a duplicate key is found in the hook dictionary.
        """
        for items in new_hook.items():
            itemA, itemB = items
            if itemA[:2] != "--":
                raise ValueError("The prefix must be '--'")
            if self._hooks.get(itemA):
                raise ValueError(
                    f"Duplicate key found: {itemA} in hook."
                    + " Please ensure unique keys."
                )

        self._hooks = {**self._hooks, **new_hook}

    def help(self) -> None:
        """Prints custom help message for Command class."""
        from colorama import Fore, Style, init

        init()

        for command, hook in self._hooks.items():
            doc = getattr(self, hook).__doc__
            print(Fore.CYAN, command + "\t:", Fore.YELLOW, doc, Style.RESET_ALL)
        exit(0)

    def setup(self) -> None:
        """Sets up the Django environment by validating conditions, running hooks, and initializing Django."""
        self._run_hook()
        self._django_setup()

        try:
            self.handle()
            exit(0)
        except Exception as e:
            if self.debug:
                if str(e) == "foo":
                    """
                    You should not return foo because you should not redirect the command to manager.py.
                    So in this case it will exit suddenly without any error messages
                    """
                    exit(1)
                raise ValueError(str(e))
            exit(0)

    def _run_hook(self) -> None:
        """Executes any hooks that match the command-line arguments."""
        if len(self._argv) > 1:
            for arg in self._argv:
                if "--" not in arg[:2]:
                    continue
                method_name = self._hooks.get(arg, None)
                if method_name:
                    method = getattr(self, method_name)
                    method()

    def _django_setup(self) -> None:
        """Runs the Django environment."""
        if self.use_django:
            from octo.base import get_settings_module

            os.environ.setdefault("DJANGO_SETTINGS_MODULE", get_settings_module())
            django.setup()
            self.django_run = True

    def no_error(self) -> None:
        """Prevent returning error."""
        self.debug = False

    def handle(self):
        """Here you create your own command"""
        pass



from octo.handler.cache import CloneCache
from octo.handler.clean import CleanTemplate
import shutil
import os
from octo.commands.generate.utils import read_file, write_file


class OctoLaunchClean(CleanTemplate):
    def clean_dir(self):
        dirs = [
            ".git",
        ]
        for dir in dirs:
            shutil.rmtree(os.path.join(self.project_path, dir))

    def clean_file(self):
        files = [
            "README.rst",
            "docs/index.rst",
            "LICENSE",
            "CONTRIBUTING.md",
        ]
        for file in files:
            os.remove(os.path.join(self.project_path, file))

    def clean_docs(self):
        docs = os.path.join(self.project_path, "docs/")
        for entry in os.scandir(docs):
            if entry.is_dir():
                shutil.rmtree(os.path.join(docs, entry.name))

    def clean_appfile(self):
        """Clears the values of all variables in the file."""
        app_file = os.path.join(self.project_path, "config/app.py")
        content = read_file(app_file)
        cleared_content = []
        for line in content:
            # Check if the line contains an assignment
            if "=" in line:
                # Split the line into variable and value
                parts = line.split("=", 1)
                # Keep the variable part and clear the value part
                cleared_line = parts[0].strip() + ' = ""\n'
                cleared_content.append(cleared_line)
            else:
                cleared_content.append(line)
        write_file(app_file, cleared_content)


class OctoLaunch(CloneCache):
    project_name_cache = "octo-launch"
    repo_url = "https://github.com/oc-to/octo-launch"
    clean_class = OctoLaunchClean



def read_file(file_path):
    """Reads the content of the file."""
    with open(file_path, "r") as file:
        content = file.readlines()
    return content


def write_file(file_path, content):
    """Writes the content to the file."""
    with open(file_path, "w") as file:
        file.writelines(content)



from PIL import Image
from io import BytesIO
from django.core.files.uploadedfile import SimpleUploadedFile
import uuid


def _exif_transpose(img):
    exif_orientation_tag = 274
    if hasattr(img, "_getexif"):
        exif_data = img._getexif()
        if exif_data is not None:
            orientation = exif_data.get(exif_orientation_tag, 1)
            orientation_transpose_map = {
                2: Image.FLIP_LEFT_RIGHT,
                3: Image.ROTATE_180,
                4: Image.FLIP_TOP_BOTTOM,
                5: Image.TRANSPOSE,
                6: Image.ROTATE_270,
                7: Image.TRANSVERSE,
                8: Image.ROTATE_90,
            }
            if orientation in orientation_transpose_map:
                img = img.transpose(orientation_transpose_map[orientation])
    return img


def resize_image(avatar, w, h):
    try:
        image = Image.open(avatar)
        image = _exif_transpose(image)
        size_avatar = (w, h)
        avatar_new_size = image.resize(size_avatar, Image.LANCZOS)
        image.close()

        avatar_new = BytesIO()
        avatar_new_size.save(avatar_new, format="PNG")
        avatar_new.seek(0)

        name = uuid.uuid4().hex[:10] + ".png"
        return SimpleUploadedFile(name, avatar_new.getvalue(), content_type="image/png")
    except Exception as e:
        raise Exception(f"Error in resizing image: {e}")



from appdirs import user_cache_dir
import os
import sys
from octo.base import _error_structure
from requests.exceptions import HTTPError
import shutil
from colorama import Fore, Style, init
from octo.handler.clean import CleanTemplate
from octo.handler.github import GitHubRepositorie


class CloneCache:
    """Caching for faster project creation"""

    cache_dir: str = None
    project_name_cache: str = None
    project_name: str = None
    repo_url: str = None
    clean_class: CleanTemplate = None

    def __init__(self, project_name: str | None) -> None:
        """Initialize the CloneCache instance with a project name
        and setup the cache directory and repository"""
        self.get_cache_dir()
        self.project_name = project_name
        self.get_repo()

    def get_cache_dir(self):
        """Get the cache directory path for the application
        and create it if it does not exist"""
        cache_dir = user_cache_dir(appname="octo_framework", appauthor=False)

        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)

        self.cache_dir = cache_dir

    def clone(self):
        """Clone the repository if it does not exist in the cache"""
        clone = False
        project_cache = os.path.join(self.cache_dir, self.project_name_cache)

        if os.path.exists(project_cache):
            # Check if the cached version is up-to-date
            clone = self.get_verion()

        if not clone:
            # If not up-to-date, remove the old cache and clone anew
            if os.path.exists(project_cache):
                shutil.rmtree(project_cache)

            self.repo.clone()

        self.clone_from_cache()

    def get_repo(self) -> GitHubRepositorie:
        """Create a GitHub repository object for the project"""
        repo = GitHubRepositorie(
            project_name=self.project_name_cache,
            repo_url=self.repo_url,
            output_path=self.cache_dir,
        )
        self.repo = repo

    def get_verion(self):
        """Check the version of the cached repository against the latest release"""
        path = os.path.join(self.cache_dir, self.project_name_cache + "/config")

        sys.path.append(path)
        try:
            from app import version  # type: ignore

            release = self.repo.get_latest_release_tag()
            return release == version
        except ImportError:
            raise ValueError(_error_structure)
        except HTTPError:
            return True

    @staticmethod
    def validate_path(output_path, dir_name):
        """Validate the output path to ensure it does not already exist"""
        if os.path.exists(output_path):
            raise ValueError(
                Fore.RED
                + Style.BRIGHT
                + "\n"
                + f"The directory {dir_name} already exists. Please choose a different project name."
                + Style.RESET_ALL
            )

    def clone_from_cache(self):
        """Clone from the cache directory to the current working directory."""
        init()

        repo_cache = os.path.join(self.cache_dir, self.project_name_cache)
        new_path = os.path.join(os.getcwd(), self.project_name)

        self.validate_path(new_path, self.project_name)

        shutil.copytree(repo_cache, new_path)
        self.clean_class(project_path=new_path).clean()
        print(
            Fore.MAGENTA
            + Style.BRIGHT
            + "\n"
            + f"A {self.project_name} has been created"
            + Style.RESET_ALL
        )



from octo.handler.manager import Manager as ManagerTemplate
from .startapp import StartApp
from .startproject import StartProject


class Manager(ManagerTemplate):
    @staticmethod
    def validate_file(): ...

    def help(self):
        from colorama import Fore, Style, init

        init()
        print(Fore.GREEN, Style.BRIGHT, "\n[Manager]", Style.RESET_ALL)
        for command, _class in self._schema.items():
            print(
                "   " + Fore.CYAN,
                command + ":\t",
                Fore.YELLOW,
                _class.__doc__,
                Style.RESET_ALL,
            )
        exit(0)


manager = Manager()
manager.set_schema(
    {
        "startapp": StartApp,
        "startproject": StartProject,
    }
)


def main():
    try:
        import django  # noqa: F401
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc

    try:
        return manager.setup()
    except Exception as e:
        if str(e) in "foo":
            manager.help()
        else:
            print(str(e))
    return exit(0)



from pathlib import Path
import os
import sys


# === variables =====================================
BASE_DIR = Path(__file__).resolve().parent

# === error message ===
_error_structure = "You can only use this with the octo-launch template or any project created with the same structure."


# === functions =====================================
def get_settings_module():
    terminal_dir = os.getcwd()

    if terminal_dir not in sys.path:
        sys.path.append(terminal_dir)

    try:
        from config.settings.base import DEBUG  # type: ignore
    except ImportError:
        raise ValueError(_error_structure)

    if DEBUG:
        return "config.settings.development"
    return "config.settings.production"



import requests
from django.contrib.auth import get_user_model
from octo.utils.generate import generate_code
from octo.logging import Logger, sanitize_message
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework.authtoken.models import Token

User = get_user_model()


class OAuthHandler:
    EMAIL_FIELD = "email"
    NAME_FIELD = "name"

    def __init__(self, access_token) -> None:
        self.logger = Logger("oauth").get()
        self.access_token = access_token
        self.service_url = self.get_service_url()
        self.token_list = {"jwt": self.generate_jwt_token, "token": self.generate_token}

    def get_service_url(self) -> str:
        """Return the URL of the service to be used for processing."""
        pass

    def validate_access_token(self) -> bool:
        """
        Validate the access token to ensure it originated from our application.

        This method checks if the provided access token was generated by our application
        and not by any other application. It returns True if the token is valid and
        originates from our application, otherwise it returns False.
        """
        pass

    def _get_headers(self):
        return {"Authorization": "Bearer " + self.access_token}

    def _fetch_data(self) -> dict | None:
        try:
            response = requests.get(self.service_url, headers=self._get_headers())
            response.raise_for_status()
            return response.json()

        except requests.RequestException as e:
            self.logger.error(sanitize_message("Failed to fetch user info: " + str(e)))
        return None

    def _get_user(self):
        user_info = self._fetch_data()

        if not user_info:
            return None

        data_user = self.get_data_user(user_info)
        return self.get_user(data_user)

    def get_data_user(self, user_info: dict) -> dict:
        email = str(user_info.get(self.EMAIL_FIELD, ""))
        username = str(email.split("@")[0] + "_" + generate_code(4))
        name = str(user_info.get(self.NAME_FIELD, username))

        return {
            "email": email,
            "username": username,
            "name": name,
        }

    def get_user(self, data_user: dict):
        try:
            email = data_user["email"]

            if not email:
                return None

            user, _ = User.objects.get_or_create(email=email, defaults=data_user)
            return user
        except Exception as e:
            self.logger.error(
                sanitize_message("Error creating or retrieving user:" + str(e))
            )
            return None

    def validate_token(self, token_type):
        if token_type not in self.token_list:
            raise ValueError(
                f"Invalid token_type. Allowed values are: {list(self.token_list.keys())}"
            )

    def _generate(self, token_type: str):
        self.validate_token(token_type)
        if self.validate_access_token():
            user = self._get_user()
            if user:
                token_list = self.token_list.get(token_type)
                return token_list(user)
        return None

    def generate_jwt_token(self, user) -> dict:
        refresh = RefreshToken.for_user(user)
        return {
            "refresh": str(refresh),
            "access": str(refresh.access_token),
        }

    def generate_token(self, user) -> dict:
        token, _ = Token.objects.get_or_create(user=user)
        return {"token": token}

    def generate(self, token_type: str):
        return self._generate(token_type)



import git
import os
import requests
import re
from colorama import Fore, Style, init


class GitHubRepositorie:
    """
    A class to handle operations related to GitHub repositories such as cloning
    and fetching repository details via GitHub API.
    """

    output_path: str = None
    project_name: str = None
    repo_url: str = None
    repo_data: dict = None

    def __init__(
        self,
        project_name: str | None = None,
        repo_url: str | None = None,
        output_path: str | None = None,
    ) -> None:
        """
        Initialize the GitHubRepositorie instance.

        Args:
            project_name (str | None): The name of the project to be created.
            repo_url (str | None): The URL of the GitHub repository.
            output_path (str | None): The directory where the repository should be cloned.
        """
        self.get_project_name(name=project_name)
        self.set_repo_url(repo_url)
        self.set_output_path(output_path)
        init()

    def get_latest_release_tag(self) -> str:
        """
        Fetch the latest release tag from the GitHub API.

        Returns:
            str: The latest release tag or None if no tag is found.
        """
        if not self._repo_url:
            raise ValueError(
                Fore.RED
                + Style.BRIGHT
                + "Repository URL has not been set."
                + Style.RESET_ALL
            )

        api_url = (
            self._repo_url.replace(
                "https://github.com/", "https://api.github.com/repos/"
            )
            + "/releases/latest"
        )
        response = requests.get(api_url)

        if response.status_code == 404:
            return None

        response.raise_for_status()
        return response.json().get("tag_name")

    def get_repo_data(self) -> str:
        """
        Fetch repository data from the GitHub API.
        """
        api_url = self._repo_url.replace(
            "https://github.com/", "https://api.github.com/repos/"
        )
        response = requests.get(api_url)

        try:
            response.raise_for_status()
            self.repo_data = response.json()
        except requests.exceptions.HTTPError as e:
            if response.status_code == 404:
                raise ValueError(
                    Fore.RED
                    + "GitHub repository not found. Please check the repository URL."
                    + Style.RESET_ALL
                )
            else:
                raise ValueError(e)

    @staticmethod
    def validate_path(output_path, dir_name):
        # Validate the output path to ensure it does not already exist
        if os.path.exists(output_path):
            raise ValueError(
                Fore.RED
                + Style.BRIGHT
                + "\n"
                + f"The directory {dir_name} already exists. Please choose a different project name."
                + Style.RESET_ALL
            )

    def clone(self):
        """
        Clone the repository from GitHub.
        """
        output_path = os.path.join(self.output_path, self.project_name)
        self.validate_path(output_path, self.project_name)

        if not self.repo_data:
            self.get_repo_data()

        latest_tag = self.get_latest_release_tag()

        if not latest_tag:
            branch = self.repo_data.get("default_branch", "main")
        else:
            branch = latest_tag

        print(
            Fore.GREEN
            + Style.BRIGHT
            + "\n"
            + f"Cloning {self.project_name}"
            + Fore.CYAN
            + "\n"
            + f"version: [{branch}]"
            + Style.RESET_ALL
            + "\n"
        )

        try:
            git.Repo.clone_from(self._repo_url, output_path, branch=branch)
            print(
                Fore.GREEN
                + Style.BRIGHT
                + self.project_name
                + " has been installed"
                + Style.RESET_ALL
                + "\n"
            )
        except Exception as e:
            print(
                Fore.RED
                + f"An error occurred while cloning the repository: {e}"
                + Style.RESET_ALL
            )

    def get_project_name(self, name: str | None = None):
        """
        Get the project name from the user if not provided.

        Args:
            name (str | None): The name of the project to be created.
        """
        self.project_name = name

        if not self.project_name:
            self.project_name = input(Fore.YELLOW + "Project Name: " + Style.RESET_ALL)

            if not self.project_name:
                self.get_project_name()

    def set_repo_url(self, repo_url: str | None):
        """
        Set the repository URL after validating it.

        Args:
            repo_url (str): The repository URL to be set.

        Raises:
            ValueError: If the URL is not a valid GitHub repository URL.
        """
        first_invalid = False
        while not repo_url or not self.is_valid_github_url(repo_url):
            if first_invalid:
                print(Fore.RED + "Invalid GitHub repository URL" + Style.RESET_ALL)
            first_invalid = True
            repo_url = input(Fore.YELLOW + "Repo URL: " + Style.RESET_ALL)

        self._repo_url = repo_url

    def set_output_path(self, path: str | None = None):
        """
        Set the output path where the repository will be cloned.

        Args:
            path (str | None): The output path to be set.
        """
        if path:
            if not os.path.exists(path):
                msg = "The path appears to be incorrect. Please verify that the path is correct"
                raise ValueError(msg)
            self.output_path = path
        else:
            self.output_path = os.getcwd()

    @staticmethod
    def is_valid_github_url(url: str) -> bool:
        """
        Validate the given GitHub repository URL.

        Args:
            url (str): The URL to be validated.

        Returns:
            bool: True if the URL is valid, False otherwise.
        """
        pattern = re.compile(
            r"^(https:\/\/github\.com\/)" r"[A-Za-z0-9_.-]+\/" r"[A-Za-z0-9_.-]+\/?$"
        )
        return bool(pattern.match(url))



from faker import Faker
from PIL import Image
from io import BytesIO
import uuid
import random
import secrets
import string
from django.core.files.uploadedfile import SimpleUploadedFile
import binascii
import os


def get_login(data: dict):
    return {
        "email": data.get("email"),
        "password": data.get("password"),
    }


def generate_register():
    fake = Faker()

    password = generate_password(100)
    return {
        "name": fake.name(),
        "username": fake.user_name()[:10],
        "email": fake.email(),
        "password": password,
        "password2": password,
    }


def generate_password(length=15):
    """I didn't use Faker to avoid password guessing"""
    alphabet = "#@$#@$#@$#@$#@$#@$" + string.ascii_letters + string.digits
    password = "".join(secrets.choice(alphabet) for i in range(length))
    return password


def remove_email(email: str):
    return email.split("@")[0] + "_" + generate_code(4)


def generate_image(width=500, height=500, color=None):
    if color is None:
        color = (
            random.randint(0, 255),
            random.randint(0, 255),
            random.randint(0, 255),
        )

    name = uuid.uuid4().hex[:10] + ".png"
    image = Image.new("RGB", (width, height), color)

    byte_io = BytesIO()
    image.save(byte_io, format="PNG")
    byte_io.seek(0)

    return SimpleUploadedFile(name, byte_io.getvalue(), content_type="image/png")


def generate_code(length=6):
    characters = string.ascii_lowercase + string.digits
    code = "".join(secrets.choice(characters) for _ in range(length))
    return code


def generate_key():
    return binascii.hexlify(os.urandom(20)).decode()



import threading


class CleanTemplate:
    """This class is responsible for performing all cleaning operations."""

    def __init__(self, project_path: str) -> None:
        self.project_path = project_path

    def clean(self) -> None:
        """Method to execute all cleaning functions in the class."""
        members = dir(self)
        threads = []

        for member in members:
            if member.startswith("clean_"):
                validate_function = getattr(self, member)
                thread = threading.Thread(target=validate_function)
                threads.append(thread)
                thread.start()

        # Wait for all threads to complete
        for thread in threads:
            thread.join()



import logging
import logging.config
from octo.logging.config import get_logging
import re


class Logger:
    def __init__(self, namefile: str | None = None) -> None:
        self._loging_name, self._config = get_logging(name=namefile)

    def get(self) -> logging:
        logging.config.dictConfig(self._config)
        return logging.getLogger(self._loging_name)


def sanitize_message(message: str) -> str:
    """Sanitize the message by removing newlines, tabs,
    and replacing them with spaces."""
    return re.sub(r"[\n\r\t]", " ", message)



from octo.handler.command import Command
from octo.commands.generate.repositories.octo_launch import OctoLaunch
from octo.handler.cache import CloneCache


templates = {
    "octo-launch": OctoLaunch,
}


class StartProject(Command):
    """Create a project from existing templates"""

    def __init__(self):
        super().__init__()
        self.use_django = False
        self.template = "octo-launch"
        self.set_hooks(
            {
                "--octo-launch": "set_octo_launch",
            }
        )

    def handle(self):
        # === import =====================================
        from colorama import Fore, Style, init

        init()
        # === Logic ======================================

        if len(self._argv) > 2:
            project_name = self._argv[2]
            template = templates.get(self.template)

            assert issubclass(template, CloneCache)
            template(project_name=str(project_name)).clone()

        else:
            print(
                Fore.RED
                + "\n"
                + "You must enter the project name"
                + Fore.GREEN
                + "\n"
                + "octo startproject project_name"
                + Style.RESET_ALL
            )

    def set_octo_launch(self):
        """Selects the octo-launch template, which is the default option"""
        self.template = "octo-launch"



import sys
from octo.handler.command import Command


class Manager:
    """The Manager class is responsible for modifying or adding some features in the manage.py file in Django."""

    _schema = {}

    def __init__(self) -> None:
        self._argv = sys.argv

    def validate(self) -> None:
        """Validates all the necessary conditions by executing methods starting with 'validate_'."""
        members = dir(self)
        for member in members:
            if member.startswith("validate_"):
                validate_function = getattr(self, member)
                validate_function()

    def validate_file(self) -> None:
        """Ensures that this script is being executed from manage.py file."""
        if "manage.py" not in self._argv[0]:
            raise ValueError("You can only use this on the manage.py file")

    def validate_schema(self) -> None:
        """Validates that the _schema attribute is a dictionary."""
        if not isinstance(self._schema, dict):
            raise ValueError("Invalid _schema format. Expected a dictionary.")

    def set_schema(self, new_schema: dict) -> None:
        """Adds new commands to the schema after checking for duplicates.

        Args:
            new_schema (dict): A dictionary of new commands to add to the schema.

        Raises:
            ValueError: If a duplicate key is found in the schema.
        """
        for items in new_schema.items():
            itemA, itemB = items
            if self._schema.get(itemA):
                raise ValueError(
                    f"Duplicate key found: {itemA} in schema."
                    + " Please ensure unique keys."
                )
            if not issubclass(itemB, Command):
                raise ValueError("This class must be an instance of Command")
        self._schema = {**self._schema, **new_schema}

    def setup(self) -> None:
        """Sets up the Django environment by validating conditions, running hooks, and initializing Django."""
        self.validate()
        self.run()

    def help(self):
        from django.core.management import execute_from_command_line
        from colorama import Fore, Style, init

        init()
        execute_from_command_line(self._argv)
        print(Fore.GREEN, Style.BRIGHT, "\n[Manager]", Style.RESET_ALL)
        for command, _class in self._schema.items():
            print(
                "   " + Fore.CYAN,
                command + ":\t",
                Fore.YELLOW,
                _class.__doc__,
                Style.RESET_ALL,
            )
        exit(0)

    def run(self) -> None:
        """Executes the specified command based on the provided arguments.

        Raises:
            ValueError: If no valid command is provided.
        """
        argv = self._argv
        help_fields = ["help", "-h", "--help"]
        if len(argv) > 1:
            if argv[1] in help_fields:
                self.help()
                raise ValueError("foo")

            method = self._schema.get(argv[1], None)

            if method:
                for arg in self._argv:
                    if arg in help_fields:
                        method().help()
                method().setup()
        else:
            self.help()
        raise ValueError("foo")



from django.conf import settings
import os

try:
    OCTO_LOGGER_NAME = settings.OCTO_LOGGER_NAME
except AttributeError:
    OCTO_LOGGER_NAME = "octo-log"


def get_logging(name: str | None = OCTO_LOGGER_NAME):
    return name, {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "colored": {
                "()": "colorlog.ColoredFormatter",
                "format": "%(log_color)s[%(asctime)s] [%(process)d] [%(module)s] [%(levelname)s] %(message)s %(reset)s",
                "datefmt": "%Y-%m-%d %H:%M:%S",
                "log_colors": {
                    "DEBUG": "cyan",
                    "INFO": "green",
                    "WARNING": "yellow",
                    "ERROR": "red",
                    "CRITICAL": "bold_red",
                },
                "style": "%",
            },
        },
        "handlers": {
            f"{name}": {
                "level": "DEBUG",
                "class": "logging.FileHandler",
                "filename": os.path.join(settings.BASE_DIR, f"logs/{name}.log"),
                "formatter": "colored",
            },
        },
        "loggers": {
            f"{name}": {
                "handlers": [f"{name}"],
                "level": "DEBUG",
                "propagate": True,
            },
        },
    }
